/**
 * Copyright (c) 2019-2021 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Viewport } from '../../mol-canvas3d/camera/util';
import { ImagePass } from '../../mol-canvas3d/passes/image';
import { Color } from '../../mol-util/color';
import { ParamDefinition as PD } from '../../mol-util/param-definition';
import { LightPlugin } from '../index';
export declare const ViewportScreenshotResolutionOptions: string[][];
export declare class ViewportScreenshotHelper {
    private plugin;
    constructor(plugin: LightPlugin);
    private ev;
    private createParams;
    private _params;
    get params(): {
        resolution: PD.Mapped<PD.NamedParams<PD.Normalize<{
            width: number;
            height: number;
        }>, "custom"> | PD.NamedParams<PD.Normalize<unknown>, "viewport"> | PD.NamedParams<PD.Normalize<unknown>, "ultra-hd"> | PD.NamedParams<PD.Normalize<unknown>, "hd"> | PD.NamedParams<PD.Normalize<unknown>, "full-hd">>;
        transparent: PD.BooleanParam;
        axes: PD.Mapped<PD.NamedParams<PD.Normalize<unknown>, "off"> | PD.NamedParams<PD.Normalize<{
            alpha: number;
            colorX: Color;
            colorY: Color;
            colorZ: Color;
            scale: number;
            location: "bottom-left" | "bottom-right" | "top-left" | "top-right";
            locationOffsetX: number;
            locationOffsetY: number;
            originColor: Color;
            radiusScale: number;
            showPlanes: boolean;
            planeColorXY: Color;
            planeColorXZ: Color;
            planeColorYZ: Color;
            showLabels: boolean;
            labelX: string;
            labelY: string;
            labelZ: string;
            labelColorX: Color;
            labelColorY: Color;
            labelColorZ: Color;
            labelOpacity: number;
            labelScale: number;
        }>, "on">>;
    };
    readonly behaviors: {
        values: import("rxjs").BehaviorSubject<PD.Values<{
            resolution: PD.Mapped<PD.NamedParams<PD.Normalize<unknown>, "viewport"> | PD.NamedParams<PD.Normalize<unknown>, "ultra-hd"> | PD.NamedParams<PD.Normalize<unknown>, "hd"> | PD.NamedParams<PD.Normalize<unknown>, "full-hd"> | PD.NamedParams<PD.Normalize<{
                width: number;
                height: number;
            }>, "custom">>;
            transparent: PD.BooleanParam;
            axes: PD.Mapped<PD.NamedParams<PD.Normalize<unknown>, "off"> | PD.NamedParams<PD.Normalize<{
                alpha: number;
                colorX: Color;
                colorY: Color;
                colorZ: Color;
                scale: number;
                location: "bottom-left" | "bottom-right" | "top-left" | "top-right";
                locationOffsetX: number;
                locationOffsetY: number;
                originColor: Color;
                radiusScale: number;
                showPlanes: boolean;
                planeColorXY: Color;
                planeColorXZ: Color;
                planeColorYZ: Color;
                showLabels: boolean;
                labelX: string;
                labelY: string;
                labelZ: string;
                labelColorX: Color;
                labelColorY: Color;
                labelColorZ: Color;
                labelOpacity: number;
                labelScale: number;
            }>, "on">>;
        }>>;
        cropParams: import("rxjs").BehaviorSubject<{
            auto: boolean;
            relativePadding: number;
        }>;
        relativeCrop: import("rxjs").BehaviorSubject<Viewport>;
    };
    readonly events: {
        previewed: import("rxjs").Subject<any>;
    };
    get values(): PD.Values<{
        resolution: PD.Mapped<PD.NamedParams<PD.Normalize<unknown>, "viewport"> | PD.NamedParams<PD.Normalize<unknown>, "ultra-hd"> | PD.NamedParams<PD.Normalize<unknown>, "hd"> | PD.NamedParams<PD.Normalize<unknown>, "full-hd"> | PD.NamedParams<PD.Normalize<{
            width: number;
            height: number;
        }>, "custom">>;
        transparent: PD.BooleanParam;
        axes: PD.Mapped<PD.NamedParams<PD.Normalize<unknown>, "off"> | PD.NamedParams<PD.Normalize<{
            alpha: number;
            colorX: Color;
            colorY: Color;
            colorZ: Color;
            scale: number;
            location: "bottom-left" | "bottom-right" | "top-left" | "top-right";
            locationOffsetX: number;
            locationOffsetY: number;
            originColor: Color;
            radiusScale: number;
            showPlanes: boolean;
            planeColorXY: Color;
            planeColorXZ: Color;
            planeColorYZ: Color;
            showLabels: boolean;
            labelX: string;
            labelY: string;
            labelZ: string;
            labelColorX: Color;
            labelColorY: Color;
            labelColorZ: Color;
            labelOpacity: number;
            labelScale: number;
        }>, "on">>;
    }>;
    get cropParams(): {
        auto: boolean;
        relativePadding: number;
    };
    get relativeCrop(): Viewport;
    private getCanvasSize;
    private getSize;
    private createPass;
    private _previewPass?;
    private get previewPass();
    private _imagePass?;
    get imagePass(): ImagePass;
    private canvas;
    private previewCanvas;
    private previewData;
    resetCrop(): void;
    toggleAutocrop(): void;
    get isFullFrame(): boolean;
    autocrop(relativePadding?: number): void;
    getPreview(maxDim?: number): {
        canvas: HTMLCanvasElement;
        width: number;
        height: number;
    } | undefined;
    getSizeAndViewport(): {
        width: number;
        height: number;
        viewport: Viewport;
    };
    private draw;
    getImageDataUri(): Promise<string>;
    copyToClipboard(): Promise<void>;
    getAxesScale(): 0.5 | 0.33 | 0.22 | 0.65;
    download(filename: string): Promise<void>;
}
